#include "HelloWorldScene.h"
#include "AudioEngine.h"
#include <chrono>

USING_NS_CC;
using namespace std::chrono;

Scene* HelloWorld::createScene()
{
    // 'scene' is an autorelease object
    auto scene = Scene::create();
    
    // 'layer' is an autorelease object
    auto layer = HelloWorld::create();

    // add layer as a child to scene
    scene->addChild(layer);

    // return the scene
    return scene;
}

// on "init" you need to initialize your instance
bool HelloWorld::init()
{
    //////////////////////////////
    // 1. super init first
    if ( !Layer::init() )
    {
        return false;
    }

    FileUtils::getInstance()->addSearchPath("sounds_wav");
    FileUtils::getInstance()->addSearchPath("sounds_ogg");

    loaded = 0;
    extension = "ogg";

    files.push_back("background1");
    files.push_back("background2");
    files.push_back("background3");
    files.push_back("background4");
    files.push_back("background5");
    files.push_back("background6");
    files.push_back("background7");
    files.push_back("background8");
    files.push_back("background9");
    files.push_back("background10");

    auto visibleSize = Director::getInstance()->getVisibleSize();
    Vec2 origin = Director::getInstance()->getVisibleOrigin();

    btnPreloadSounds = ui::Button::create("HelloWorld.png");
    btnPreloadSounds->setPosition(Vec2(visibleSize.width * 0.25f + origin.x, visibleSize.height * 0.75f + origin.y));
    btnPreloadSounds->setTitleText("Preload sounds");
    btnPreloadSounds->addTouchEventListener(CC_CALLBACK_2(HelloWorld::preloadSounds, this));
    addChild(btnPreloadSounds);

    btnUnloadSounds = ui::Button::create("HelloWorld.png");
    btnUnloadSounds->setPosition(Vec2(visibleSize.width * 0.25f + origin.x, visibleSize.height * 0.25f + origin.y));
    btnUnloadSounds->setTitleText("Unload sounds");
    btnUnloadSounds->addTouchEventListener(CC_CALLBACK_2(HelloWorld::unloadSounds, this));
    addChild(btnUnloadSounds);

    btnPlayLastSound = ui::Button::create("HelloWorld.png");
    btnPlayLastSound->setPosition(Vec2(visibleSize.width * 0.75f + origin.x, visibleSize.height * 0.25f + origin.y));
    btnPlayLastSound->setTitleText("Play Last Sound");
    addChild(btnPlayLastSound);
    btnPlayLastSound->addTouchEventListener([&](Ref* ref, ui::Widget::TouchEventType touchEventType) {
        if (touchEventType != cocos2d::ui::Widget::TouchEventType::ENDED) return;
        if(loaded < files.size()){
            CCLOG("not yet loaded!");
            return;
        }
        std::string filename = files.at(files.size() - 1) + "." + extension;
        cocos2d::experimental::AudioEngine::play2d(filename);
    });

    btnToggleExtension = ui::Button::create("HelloWorld.png");
    btnToggleExtension->setPosition(Vec2(visibleSize.width * 0.75f + origin.x, visibleSize.height * 0.75f + origin.y));
    btnToggleExtension->setTitleText("Switch to WAV");
    btnToggleExtension->addTouchEventListener([&](Ref* ref, ui::Widget::TouchEventType touchEventType) {
        if (touchEventType != cocos2d::ui::Widget::TouchEventType::ENDED) return;
        if(loaded == files.size()){
            unloadSounds(ref, touchEventType);
        }
        if(loaded == 0){
            if(extension == "ogg"){
                extension = "wav";
                btnToggleExtension->setTitleText("Switch To OGG");
            }
            else{
                extension = "ogg";
                btnToggleExtension->setTitleText("Switch To WAV");
            }
        }
        else{
            CCLOG("something's still loading. Cannot change extension.");
        }

    });
    addChild(btnToggleExtension);
    
    return true;
}

void HelloWorld::preloadSounds(Ref* ref, ui::Widget::TouchEventType touchEventType) {
    if(touchEventType != ui::Widget::TouchEventType::ENDED) return;
    if(loaded > 0) return;
    CCLOG("Started preloading!");
    milliseconds startMiliseconds = duration_cast< milliseconds >(
            system_clock::now().time_since_epoch()
    );
    for(int i = 0; i < files.size(); i++){
        std::string filename = files.at(i) + "." + extension;
        const int j = i;
        cocos2d::experimental::AudioEngine::preload(filename, [&, filename, startMiliseconds, j](bool success){
            CCLOG("%s done loading. Result: %s", filename.c_str(), success ? "success" : "failure");
            loaded++;
            if(loaded == files.size()){
                CCLOG("All done!");
                milliseconds endMiliseconds = duration_cast< milliseconds >(
                        system_clock::now().time_since_epoch()
                );
                milliseconds diff = endMiliseconds - startMiliseconds;
                CCLOG("loading took: %lld ms", diff.count());
            }
        });
        if(i == 0){
            cocos2d::experimental::AudioEngine::play2d(filename); //this causes very long loading
        }
    }
}

void HelloWorld::unloadSounds(Ref* ref, ui::Widget::TouchEventType touchEventType) {
    if(touchEventType != ui::Widget::TouchEventType::ENDED) return;
    if(loaded < files.size()) return;
    CCLOG("Started unloading!");
    for(int i = 0; i < files.size(); i++){
        std::string filename = files.at(i) + "." + extension;
        cocos2d::experimental::AudioEngine::uncache(filename);
        loaded--;
    }
}

std::string to_str(int t) {
    std::stringstream ss;
    ss << t;
    std::string str = ss.str();
    return str;
}